from AutoRPE.UtilsRPE.SourceManager import load_list_of_files_to_keep_unmodified, load_vault, \
    preprocess_sources, postprocess_sources, parse_sources
from AutoRPE.UtilsRPE.Inserter import add_sbits_to_rpe_variables, insert_read_precisions_module_to_file
from os import mkdir, rename
from os.path import isdir
from shutil import rmtree


def get_command_line_arguments():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--input-source", default=None,
                        help="Path to a folder that contains the preprocessed code source")
    parser.add_argument("-o", "--output-vault", default="vault.pkl",
                        help="Filename of the output database")

    args = parser.parse_args()

    if args.input_source is None:
        parser.error("Argument -i/--input-sources required!")

    if not isdir(args.input_source):
        parser.error("Argument -v/--vault %s is not a file." % args.vault)

    return args


if __name__ == "__main__":

    args = get_command_line_arguments()

    path_to_input_sources = args.input_source
    output_vault = args.output_vault

    # Paths to source folders
    path_to_preprocessed_sources = "./tmp_processedsources/"
    path_to_output_sources = "./tmp_outputsources/"

    if isdir(path_to_preprocessed_sources):
        rmtree(path_to_preprocessed_sources)
    if isdir(path_to_output_sources):
        rmtree(path_to_output_sources)

    mkdir(path_to_output_sources)
    mkdir(path_to_preprocessed_sources)

    print("Starting the implementation of the reduced precision emulator in:\n   %s\n\n" % path_to_input_sources)

    # List of files that should not be processed, as red from "list_of_files_to_keep_unmodified.txt"
    preprocess_blacklist = load_list_of_files_to_keep_unmodified()

    # Format text + replace real declarations with type(rpe_var)
    preprocess_sources(path_to_input_sources, path_to_preprocessed_sources, blacklist=preprocess_blacklist)

    # Obtain source information
    vault = parse_sources(path_to_preprocessed_sources, save_vault=True)

    # We load the code information into the vault
    try:
        vault
    except NameError:
        vault = load_vault()

    # Process the sources to fix call to functions, read/write, ...
    postprocess_sources(path_to_preprocessed_sources, path_to_output_sources, vault=vault)

    # Insert precision specification
    add_sbits_to_rpe_variables(path_to_output_sources, vault=vault)

    # Assign ids to variables that still don't have any (can't have it's precision changed online)
    # and set them as mutable=False
    vault.assign_ids_to_immutable_variables()

    # Add the read_precisions module into a file (should be accessible for all the other modules)
    file_to_add_read_precisions_module = None
    insert_read_precisions_module_to_file(path_to_output_sources, file_to_add_read_precisions_module,
                                          vault=vault)

    # Deleting temporary folders
    if isdir(path_to_preprocessed_sources):
        rmtree(path_to_preprocessed_sources)

    output_folder = "SourcesWithRPE"
    if isdir(output_folder):
        rmtree(output_folder)
    if isdir(path_to_output_sources):
        rename(path_to_output_sources, output_folder)
